/* x86 opcode map generated from x86-opcode-map.txt */
/* Do not change this code. */

/* Table: one byte opcode */
const insn_attr_t inat_primary_table[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_MODRM,
	[0x01] = INAT_MODRM,
	[0x02] = INAT_MODRM,
	[0x03] = INAT_MODRM,
	[0x04] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x05] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x08] = INAT_MODRM,
	[0x09] = INAT_MODRM,
	[0x0a] = INAT_MODRM,
	[0x0b] = INAT_MODRM,
	[0x0c] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x0d] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x0f] = INAT_MAKE_ESCAPE(1),
	[0x10] = INAT_MODRM,
	[0x11] = INAT_MODRM,
	[0x12] = INAT_MODRM,
	[0x13] = INAT_MODRM,
	[0x14] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x15] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x18] = INAT_MODRM,
	[0x19] = INAT_MODRM,
	[0x1a] = INAT_MODRM,
	[0x1b] = INAT_MODRM,
	[0x1c] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x1d] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x20] = INAT_MODRM,
	[0x21] = INAT_MODRM,
	[0x22] = INAT_MODRM,
	[0x23] = INAT_MODRM,
	[0x24] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x25] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x26] = INAT_MAKE_PREFIX(INAT_PFX_ES),
	[0x28] = INAT_MODRM,
	[0x29] = INAT_MODRM,
	[0x2a] = INAT_MODRM,
	[0x2b] = INAT_MODRM,
	[0x2c] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x2d] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x2e] = INAT_MAKE_PREFIX(INAT_PFX_CS),
	[0x30] = INAT_MODRM,
	[0x31] = INAT_MODRM,
	[0x32] = INAT_MODRM,
	[0x33] = INAT_MODRM,
	[0x34] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x35] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x36] = INAT_MAKE_PREFIX(INAT_PFX_SS),
	[0x38] = INAT_MODRM,
	[0x39] = INAT_MODRM,
	[0x3a] = INAT_MODRM,
	[0x3b] = INAT_MODRM,
	[0x3c] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x3d] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0x3e] = INAT_MAKE_PREFIX(INAT_PFX_DS),
	[0x40] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x41] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x42] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x43] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x44] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x45] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x46] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x47] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x48] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x49] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4a] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4b] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4c] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4d] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4e] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x4f] = INAT_MAKE_PREFIX(INAT_PFX_REX),
	[0x50] = INAT_FORCE64,
	[0x51] = INAT_FORCE64,
	[0x52] = INAT_FORCE64,
	[0x53] = INAT_FORCE64,
	[0x54] = INAT_FORCE64,
	[0x55] = INAT_FORCE64,
	[0x56] = INAT_FORCE64,
	[0x57] = INAT_FORCE64,
	[0x58] = INAT_FORCE64,
	[0x59] = INAT_FORCE64,
	[0x5a] = INAT_FORCE64,
	[0x5b] = INAT_FORCE64,
	[0x5c] = INAT_FORCE64,
	[0x5d] = INAT_FORCE64,
	[0x5e] = INAT_FORCE64,
	[0x5f] = INAT_FORCE64,
	[0x62] = INAT_MODRM,
	[0x63] = INAT_MODRM | INAT_MODRM,
	[0x64] = INAT_MAKE_PREFIX(INAT_PFX_FS),
	[0x65] = INAT_MAKE_PREFIX(INAT_PFX_GS),
	[0x66] = INAT_MAKE_PREFIX(INAT_PFX_OPNDSZ),
	[0x67] = INAT_MAKE_PREFIX(INAT_PFX_ADDRSZ),
	[0x68] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x69] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_MODRM,
	[0x6a] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0x6b] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0x70] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x71] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x72] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x73] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x74] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x75] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x76] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x77] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x78] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x79] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7a] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7b] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7c] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7d] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7e] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x7f] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0x80] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(1),
	[0x81] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_MODRM | INAT_MAKE_GROUP(1),
	[0x82] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(1),
	[0x83] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(1),
	[0x84] = INAT_MODRM,
	[0x85] = INAT_MODRM,
	[0x86] = INAT_MODRM,
	[0x87] = INAT_MODRM,
	[0x88] = INAT_MODRM,
	[0x89] = INAT_MODRM,
	[0x8a] = INAT_MODRM,
	[0x8b] = INAT_MODRM,
	[0x8c] = INAT_MODRM,
	[0x8d] = INAT_MODRM,
	[0x8e] = INAT_MODRM,
	[0x8f] = INAT_MAKE_GROUP(2) | INAT_MODRM | INAT_FORCE64,
	[0x9a] = INAT_MAKE_IMM(INAT_IMM_PTR),
	[0x9c] = INAT_FORCE64,
	[0x9d] = INAT_FORCE64,
	[0xa0] = INAT_MOFFSET,
	[0xa1] = INAT_MOFFSET,
	[0xa2] = INAT_MOFFSET,
	[0xa3] = INAT_MOFFSET,
	[0xa8] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xa9] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
	[0xb0] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb1] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb2] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb3] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb4] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb5] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb6] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb7] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xb8] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xb9] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xba] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xbb] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xbc] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xbd] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xbe] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xbf] = INAT_MAKE_IMM(INAT_IMM_VWORD),
	[0xc0] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xc1] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xc2] = INAT_MAKE_IMM(INAT_IMM_WORD) | INAT_FORCE64,
	[0xc4] = INAT_MODRM | INAT_MAKE_PREFIX(INAT_PFX_VEX3),
	[0xc5] = INAT_MODRM | INAT_MAKE_PREFIX(INAT_PFX_VEX2),
	[0xc6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(4),
	[0xc7] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_MODRM | INAT_MAKE_GROUP(5),
	[0xc8] = INAT_MAKE_IMM(INAT_IMM_WORD) | INAT_SCNDIMM,
	[0xc9] = INAT_FORCE64,
	[0xca] = INAT_MAKE_IMM(INAT_IMM_WORD),
	[0xcd] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xd0] = INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xd1] = INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xd2] = INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xd3] = INAT_MODRM | INAT_MAKE_GROUP(3),
	[0xd4] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xd5] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xd8] = INAT_MODRM,
	[0xd9] = INAT_MODRM,
	[0xda] = INAT_MODRM,
	[0xdb] = INAT_MODRM,
	[0xdc] = INAT_MODRM,
	[0xdd] = INAT_MODRM,
	[0xde] = INAT_MODRM,
	[0xdf] = INAT_MODRM,
	[0xe0] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0xe1] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0xe2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0xe3] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0xe4] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xe5] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xe6] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xe7] = INAT_MAKE_IMM(INAT_IMM_BYTE),
	[0xe8] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0xe9] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0xea] = INAT_MAKE_IMM(INAT_IMM_PTR),
	[0xeb] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_FORCE64,
	[0xf0] = INAT_MAKE_PREFIX(INAT_PFX_LOCK),
	[0xf2] = INAT_MAKE_PREFIX(INAT_PFX_REPNE) | INAT_MAKE_PREFIX(INAT_PFX_REPNE),
	[0xf3] = INAT_MAKE_PREFIX(INAT_PFX_REPE) | INAT_MAKE_PREFIX(INAT_PFX_REPE),
	[0xf6] = INAT_MODRM | INAT_MAKE_GROUP(6),
	[0xf7] = INAT_MODRM | INAT_MAKE_GROUP(7),
	[0xfe] = INAT_MAKE_GROUP(8),
	[0xff] = INAT_MAKE_GROUP(9),
};

/* Table: 2-byte opcode (0x0f) */
const insn_attr_t inat_escape_table_1[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_MAKE_GROUP(10),
	[0x01] = INAT_MAKE_GROUP(11),
	[0x02] = INAT_MODRM,
	[0x03] = INAT_MODRM,
	[0x0d] = INAT_MAKE_GROUP(12),
	[0x0f] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0x10] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x11] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x12] = INAT_MODRM | INAT_VEXOK | INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x13] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x14] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x15] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x16] = INAT_MODRM | INAT_VEXOK | INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x17] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x18] = INAT_MAKE_GROUP(13),
	[0x1a] = INAT_MODRM | INAT_MODRM | INAT_MODRM | INAT_MODRM,
	[0x1b] = INAT_MODRM | INAT_MODRM | INAT_MODRM | INAT_MODRM,
	[0x1f] = INAT_MODRM,
	[0x20] = INAT_MODRM,
	[0x21] = INAT_MODRM,
	[0x22] = INAT_MODRM,
	[0x23] = INAT_MODRM,
	[0x28] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x29] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x2a] = INAT_MODRM | INAT_VARIANT,
	[0x2b] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x2c] = INAT_MODRM | INAT_VARIANT,
	[0x2d] = INAT_MODRM | INAT_VARIANT,
	[0x2e] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x2f] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x38] = INAT_MAKE_ESCAPE(2),
	[0x3a] = INAT_MAKE_ESCAPE(3),
	[0x40] = INAT_MODRM,
	[0x41] = INAT_MODRM,
	[0x42] = INAT_MODRM,
	[0x43] = INAT_MODRM,
	[0x44] = INAT_MODRM,
	[0x45] = INAT_MODRM,
	[0x46] = INAT_MODRM,
	[0x47] = INAT_MODRM,
	[0x48] = INAT_MODRM,
	[0x49] = INAT_MODRM,
	[0x4a] = INAT_MODRM,
	[0x4b] = INAT_MODRM,
	[0x4c] = INAT_MODRM,
	[0x4d] = INAT_MODRM,
	[0x4e] = INAT_MODRM,
	[0x4f] = INAT_MODRM,
	[0x50] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x51] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x52] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x53] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x54] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x55] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x56] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x57] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x58] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x59] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5a] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5b] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5c] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5d] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5e] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x5f] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x60] = INAT_MODRM | INAT_VARIANT,
	[0x61] = INAT_MODRM | INAT_VARIANT,
	[0x62] = INAT_MODRM | INAT_VARIANT,
	[0x63] = INAT_MODRM | INAT_VARIANT,
	[0x64] = INAT_MODRM | INAT_VARIANT,
	[0x65] = INAT_MODRM | INAT_VARIANT,
	[0x66] = INAT_MODRM | INAT_VARIANT,
	[0x67] = INAT_MODRM | INAT_VARIANT,
	[0x68] = INAT_MODRM | INAT_VARIANT,
	[0x69] = INAT_MODRM | INAT_VARIANT,
	[0x6a] = INAT_MODRM | INAT_VARIANT,
	[0x6b] = INAT_MODRM | INAT_VARIANT,
	[0x6c] = INAT_VARIANT,
	[0x6d] = INAT_VARIANT,
	[0x6e] = INAT_MODRM | INAT_VARIANT,
	[0x6f] = INAT_MODRM | INAT_VARIANT,
	[0x70] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x71] = INAT_MAKE_GROUP(14),
	[0x72] = INAT_MAKE_GROUP(15),
	[0x73] = INAT_MAKE_GROUP(16),
	[0x74] = INAT_MODRM | INAT_VARIANT,
	[0x75] = INAT_MODRM | INAT_VARIANT,
	[0x76] = INAT_MODRM | INAT_VARIANT,
	[0x77] = INAT_VEXOK | INAT_VEXOK,
	[0x78] = INAT_MODRM,
	[0x79] = INAT_MODRM,
	[0x7c] = INAT_VARIANT,
	[0x7d] = INAT_VARIANT,
	[0x7e] = INAT_MODRM | INAT_VARIANT,
	[0x7f] = INAT_MODRM | INAT_VARIANT,
	[0x80] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x81] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x82] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x83] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x84] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x85] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x86] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x87] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x88] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x89] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8a] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8b] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8c] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8d] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8e] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x8f] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_FORCE64,
	[0x90] = INAT_MODRM,
	[0x91] = INAT_MODRM,
	[0x92] = INAT_MODRM,
	[0x93] = INAT_MODRM,
	[0x94] = INAT_MODRM,
	[0x95] = INAT_MODRM,
	[0x96] = INAT_MODRM,
	[0x97] = INAT_MODRM,
	[0x98] = INAT_MODRM,
	[0x99] = INAT_MODRM,
	[0x9a] = INAT_MODRM,
	[0x9b] = INAT_MODRM,
	[0x9c] = INAT_MODRM,
	[0x9d] = INAT_MODRM,
	[0x9e] = INAT_MODRM,
	[0x9f] = INAT_MODRM,
	[0xa0] = INAT_FORCE64,
	[0xa1] = INAT_FORCE64,
	[0xa3] = INAT_MODRM,
	[0xa4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0xa5] = INAT_MODRM,
	[0xa6] = INAT_MAKE_GROUP(17),
	[0xa7] = INAT_MAKE_GROUP(18),
	[0xa8] = INAT_FORCE64,
	[0xa9] = INAT_FORCE64,
	[0xab] = INAT_MODRM,
	[0xac] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0xad] = INAT_MODRM,
	[0xae] = INAT_MAKE_GROUP(19),
	[0xaf] = INAT_MODRM,
	[0xb0] = INAT_MODRM,
	[0xb1] = INAT_MODRM,
	[0xb2] = INAT_MODRM,
	[0xb3] = INAT_MODRM,
	[0xb4] = INAT_MODRM,
	[0xb5] = INAT_MODRM,
	[0xb6] = INAT_MODRM,
	[0xb7] = INAT_MODRM,
	[0xb8] = INAT_VARIANT,
	[0xb9] = INAT_MAKE_GROUP(20),
	[0xba] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_MAKE_GROUP(21),
	[0xbb] = INAT_MODRM,
	[0xbc] = INAT_MODRM | INAT_VARIANT,
	[0xbd] = INAT_MODRM | INAT_VARIANT,
	[0xbe] = INAT_MODRM,
	[0xbf] = INAT_MODRM,
	[0xc0] = INAT_MODRM,
	[0xc1] = INAT_MODRM,
	[0xc2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0xc3] = INAT_MODRM,
	[0xc4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0xc5] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0xc6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0xc7] = INAT_MAKE_GROUP(22),
	[0xd0] = INAT_VARIANT,
	[0xd1] = INAT_MODRM | INAT_VARIANT,
	[0xd2] = INAT_MODRM | INAT_VARIANT,
	[0xd3] = INAT_MODRM | INAT_VARIANT,
	[0xd4] = INAT_MODRM | INAT_VARIANT,
	[0xd5] = INAT_MODRM | INAT_VARIANT,
	[0xd6] = INAT_VARIANT,
	[0xd7] = INAT_MODRM | INAT_VARIANT,
	[0xd8] = INAT_MODRM | INAT_VARIANT,
	[0xd9] = INAT_MODRM | INAT_VARIANT,
	[0xda] = INAT_MODRM | INAT_VARIANT,
	[0xdb] = INAT_MODRM | INAT_VARIANT,
	[0xdc] = INAT_MODRM | INAT_VARIANT,
	[0xdd] = INAT_MODRM | INAT_VARIANT,
	[0xde] = INAT_MODRM | INAT_VARIANT,
	[0xdf] = INAT_MODRM | INAT_VARIANT,
	[0xe0] = INAT_MODRM | INAT_VARIANT,
	[0xe1] = INAT_MODRM | INAT_VARIANT,
	[0xe2] = INAT_MODRM | INAT_VARIANT,
	[0xe3] = INAT_MODRM | INAT_VARIANT,
	[0xe4] = INAT_MODRM | INAT_VARIANT,
	[0xe5] = INAT_MODRM | INAT_VARIANT,
	[0xe6] = INAT_VARIANT,
	[0xe7] = INAT_MODRM | INAT_VARIANT,
	[0xe8] = INAT_MODRM | INAT_VARIANT,
	[0xe9] = INAT_MODRM | INAT_VARIANT,
	[0xea] = INAT_MODRM | INAT_VARIANT,
	[0xeb] = INAT_MODRM | INAT_VARIANT,
	[0xec] = INAT_MODRM | INAT_VARIANT,
	[0xed] = INAT_MODRM | INAT_VARIANT,
	[0xee] = INAT_MODRM | INAT_VARIANT,
	[0xef] = INAT_MODRM | INAT_VARIANT,
	[0xf0] = INAT_VARIANT,
	[0xf1] = INAT_MODRM | INAT_VARIANT,
	[0xf2] = INAT_MODRM | INAT_VARIANT,
	[0xf3] = INAT_MODRM | INAT_VARIANT,
	[0xf4] = INAT_MODRM | INAT_VARIANT,
	[0xf5] = INAT_MODRM | INAT_VARIANT,
	[0xf6] = INAT_MODRM | INAT_VARIANT,
	[0xf7] = INAT_MODRM | INAT_VARIANT,
	[0xf8] = INAT_MODRM | INAT_VARIANT,
	[0xf9] = INAT_MODRM | INAT_VARIANT,
	[0xfa] = INAT_MODRM | INAT_VARIANT,
	[0xfb] = INAT_MODRM | INAT_VARIANT,
	[0xfc] = INAT_MODRM | INAT_VARIANT,
	[0xfd] = INAT_MODRM | INAT_VARIANT,
	[0xfe] = INAT_MODRM | INAT_VARIANT,
};
const insn_attr_t inat_escape_table_1_1[INAT_OPCODE_TABLE_SIZE] = {
	[0x10] = INAT_MODRM | INAT_VEXOK,
	[0x11] = INAT_MODRM | INAT_VEXOK,
	[0x12] = INAT_MODRM | INAT_VEXOK,
	[0x13] = INAT_MODRM | INAT_VEXOK,
	[0x14] = INAT_MODRM | INAT_VEXOK,
	[0x15] = INAT_MODRM | INAT_VEXOK,
	[0x16] = INAT_MODRM | INAT_VEXOK,
	[0x17] = INAT_MODRM | INAT_VEXOK,
	[0x28] = INAT_MODRM | INAT_VEXOK,
	[0x29] = INAT_MODRM | INAT_VEXOK,
	[0x2a] = INAT_MODRM,
	[0x2b] = INAT_MODRM | INAT_VEXOK,
	[0x2c] = INAT_MODRM,
	[0x2d] = INAT_MODRM,
	[0x2e] = INAT_MODRM | INAT_VEXOK,
	[0x2f] = INAT_MODRM | INAT_VEXOK,
	[0x50] = INAT_MODRM | INAT_VEXOK,
	[0x51] = INAT_MODRM | INAT_VEXOK,
	[0x54] = INAT_MODRM | INAT_VEXOK,
	[0x55] = INAT_MODRM | INAT_VEXOK,
	[0x56] = INAT_MODRM | INAT_VEXOK,
	[0x57] = INAT_MODRM | INAT_VEXOK,
	[0x58] = INAT_MODRM | INAT_VEXOK,
	[0x59] = INAT_MODRM | INAT_VEXOK,
	[0x5a] = INAT_MODRM | INAT_VEXOK,
	[0x5b] = INAT_MODRM | INAT_VEXOK,
	[0x5c] = INAT_MODRM | INAT_VEXOK,
	[0x5d] = INAT_MODRM | INAT_VEXOK,
	[0x5e] = INAT_MODRM | INAT_VEXOK,
	[0x5f] = INAT_MODRM | INAT_VEXOK,
	[0x60] = INAT_MODRM | INAT_VEXOK,
	[0x61] = INAT_MODRM | INAT_VEXOK,
	[0x62] = INAT_MODRM | INAT_VEXOK,
	[0x63] = INAT_MODRM | INAT_VEXOK,
	[0x64] = INAT_MODRM | INAT_VEXOK,
	[0x65] = INAT_MODRM | INAT_VEXOK,
	[0x66] = INAT_MODRM | INAT_VEXOK,
	[0x67] = INAT_MODRM | INAT_VEXOK,
	[0x68] = INAT_MODRM | INAT_VEXOK,
	[0x69] = INAT_MODRM | INAT_VEXOK,
	[0x6a] = INAT_MODRM | INAT_VEXOK,
	[0x6b] = INAT_MODRM | INAT_VEXOK,
	[0x6c] = INAT_MODRM | INAT_VEXOK,
	[0x6d] = INAT_MODRM | INAT_VEXOK,
	[0x6e] = INAT_MODRM | INAT_VEXOK,
	[0x6f] = INAT_MODRM | INAT_VEXOK,
	[0x70] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x74] = INAT_MODRM | INAT_VEXOK,
	[0x75] = INAT_MODRM | INAT_VEXOK,
	[0x76] = INAT_MODRM | INAT_VEXOK,
	[0x7c] = INAT_MODRM | INAT_VEXOK,
	[0x7d] = INAT_MODRM | INAT_VEXOK,
	[0x7e] = INAT_MODRM | INAT_VEXOK,
	[0x7f] = INAT_MODRM | INAT_VEXOK,
	[0xbc] = INAT_MODRM,
	[0xbd] = INAT_MODRM,
	[0xc2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xc4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xc5] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xc6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xd0] = INAT_MODRM | INAT_VEXOK,
	[0xd1] = INAT_MODRM | INAT_VEXOK,
	[0xd2] = INAT_MODRM | INAT_VEXOK,
	[0xd3] = INAT_MODRM | INAT_VEXOK,
	[0xd4] = INAT_MODRM | INAT_VEXOK,
	[0xd5] = INAT_MODRM | INAT_VEXOK,
	[0xd6] = INAT_MODRM | INAT_VEXOK,
	[0xd7] = INAT_MODRM | INAT_VEXOK,
	[0xd8] = INAT_MODRM | INAT_VEXOK,
	[0xd9] = INAT_MODRM | INAT_VEXOK,
	[0xda] = INAT_MODRM | INAT_VEXOK,
	[0xdb] = INAT_MODRM | INAT_VEXOK,
	[0xdc] = INAT_MODRM | INAT_VEXOK,
	[0xdd] = INAT_MODRM | INAT_VEXOK,
	[0xde] = INAT_MODRM | INAT_VEXOK,
	[0xdf] = INAT_MODRM | INAT_VEXOK,
	[0xe0] = INAT_MODRM | INAT_VEXOK,
	[0xe1] = INAT_MODRM | INAT_VEXOK,
	[0xe2] = INAT_MODRM | INAT_VEXOK,
	[0xe3] = INAT_MODRM | INAT_VEXOK,
	[0xe4] = INAT_MODRM | INAT_VEXOK,
	[0xe5] = INAT_MODRM | INAT_VEXOK,
	[0xe6] = INAT_MODRM | INAT_VEXOK,
	[0xe7] = INAT_MODRM | INAT_VEXOK,
	[0xe8] = INAT_MODRM | INAT_VEXOK,
	[0xe9] = INAT_MODRM | INAT_VEXOK,
	[0xea] = INAT_MODRM | INAT_VEXOK,
	[0xeb] = INAT_MODRM | INAT_VEXOK,
	[0xec] = INAT_MODRM | INAT_VEXOK,
	[0xed] = INAT_MODRM | INAT_VEXOK,
	[0xee] = INAT_MODRM | INAT_VEXOK,
	[0xef] = INAT_MODRM | INAT_VEXOK,
	[0xf1] = INAT_MODRM | INAT_VEXOK,
	[0xf2] = INAT_MODRM | INAT_VEXOK,
	[0xf3] = INAT_MODRM | INAT_VEXOK,
	[0xf4] = INAT_MODRM | INAT_VEXOK,
	[0xf5] = INAT_MODRM | INAT_VEXOK,
	[0xf6] = INAT_MODRM | INAT_VEXOK,
	[0xf7] = INAT_MODRM | INAT_VEXOK,
	[0xf8] = INAT_MODRM | INAT_VEXOK,
	[0xf9] = INAT_MODRM | INAT_VEXOK,
	[0xfa] = INAT_MODRM | INAT_VEXOK,
	[0xfb] = INAT_MODRM | INAT_VEXOK,
	[0xfc] = INAT_MODRM | INAT_VEXOK,
	[0xfd] = INAT_MODRM | INAT_VEXOK,
	[0xfe] = INAT_MODRM | INAT_VEXOK,
};
const insn_attr_t inat_escape_table_1_2[INAT_OPCODE_TABLE_SIZE] = {
	[0x10] = INAT_MODRM | INAT_VEXOK,
	[0x11] = INAT_MODRM | INAT_VEXOK,
	[0x12] = INAT_MODRM | INAT_VEXOK,
	[0x16] = INAT_MODRM | INAT_VEXOK,
	[0x2a] = INAT_MODRM | INAT_VEXOK,
	[0x2c] = INAT_MODRM | INAT_VEXOK,
	[0x2d] = INAT_MODRM | INAT_VEXOK,
	[0x51] = INAT_MODRM | INAT_VEXOK,
	[0x52] = INAT_MODRM | INAT_VEXOK,
	[0x53] = INAT_MODRM | INAT_VEXOK,
	[0x58] = INAT_MODRM | INAT_VEXOK,
	[0x59] = INAT_MODRM | INAT_VEXOK,
	[0x5a] = INAT_MODRM | INAT_VEXOK,
	[0x5b] = INAT_MODRM | INAT_VEXOK,
	[0x5c] = INAT_MODRM | INAT_VEXOK,
	[0x5d] = INAT_MODRM | INAT_VEXOK,
	[0x5e] = INAT_MODRM | INAT_VEXOK,
	[0x5f] = INAT_MODRM | INAT_VEXOK,
	[0x6f] = INAT_MODRM | INAT_VEXOK,
	[0x70] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x7e] = INAT_MODRM | INAT_VEXOK,
	[0x7f] = INAT_MODRM | INAT_VEXOK,
	[0xb8] = INAT_MODRM,
	[0xbc] = INAT_MODRM,
	[0xbd] = INAT_MODRM,
	[0xc2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xd6] = INAT_MODRM,
	[0xe6] = INAT_MODRM | INAT_VEXOK,
};
const insn_attr_t inat_escape_table_1_3[INAT_OPCODE_TABLE_SIZE] = {
	[0x10] = INAT_MODRM | INAT_VEXOK,
	[0x11] = INAT_MODRM | INAT_VEXOK,
	[0x12] = INAT_MODRM | INAT_VEXOK,
	[0x2a] = INAT_MODRM | INAT_VEXOK,
	[0x2c] = INAT_MODRM | INAT_VEXOK,
	[0x2d] = INAT_MODRM | INAT_VEXOK,
	[0x51] = INAT_MODRM | INAT_VEXOK,
	[0x58] = INAT_MODRM | INAT_VEXOK,
	[0x59] = INAT_MODRM | INAT_VEXOK,
	[0x5a] = INAT_MODRM | INAT_VEXOK,
	[0x5c] = INAT_MODRM | INAT_VEXOK,
	[0x5d] = INAT_MODRM | INAT_VEXOK,
	[0x5e] = INAT_MODRM | INAT_VEXOK,
	[0x5f] = INAT_MODRM | INAT_VEXOK,
	[0x70] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x7c] = INAT_MODRM | INAT_VEXOK,
	[0x7d] = INAT_MODRM | INAT_VEXOK,
	[0xbc] = INAT_MODRM,
	[0xbd] = INAT_MODRM,
	[0xc2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xd0] = INAT_MODRM | INAT_VEXOK,
	[0xd6] = INAT_MODRM,
	[0xe6] = INAT_MODRM | INAT_VEXOK,
	[0xf0] = INAT_MODRM | INAT_VEXOK,
};

/* Table: 3-byte opcode 1 (0x0f 0x38) */
const insn_attr_t inat_escape_table_2[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_MODRM | INAT_VARIANT,
	[0x01] = INAT_MODRM | INAT_VARIANT,
	[0x02] = INAT_MODRM | INAT_VARIANT,
	[0x03] = INAT_MODRM | INAT_VARIANT,
	[0x04] = INAT_MODRM | INAT_VARIANT,
	[0x05] = INAT_MODRM | INAT_VARIANT,
	[0x06] = INAT_MODRM | INAT_VARIANT,
	[0x07] = INAT_MODRM | INAT_VARIANT,
	[0x08] = INAT_MODRM | INAT_VARIANT,
	[0x09] = INAT_MODRM | INAT_VARIANT,
	[0x0a] = INAT_MODRM | INAT_VARIANT,
	[0x0b] = INAT_MODRM | INAT_VARIANT,
	[0x0c] = INAT_VARIANT,
	[0x0d] = INAT_VARIANT,
	[0x0e] = INAT_VARIANT,
	[0x0f] = INAT_VARIANT,
	[0x10] = INAT_VARIANT,
	[0x13] = INAT_VARIANT,
	[0x14] = INAT_VARIANT,
	[0x15] = INAT_VARIANT,
	[0x16] = INAT_VARIANT,
	[0x17] = INAT_VARIANT,
	[0x18] = INAT_VARIANT,
	[0x19] = INAT_VARIANT,
	[0x1a] = INAT_VARIANT,
	[0x1c] = INAT_MODRM | INAT_VARIANT,
	[0x1d] = INAT_MODRM | INAT_VARIANT,
	[0x1e] = INAT_MODRM | INAT_VARIANT,
	[0x20] = INAT_VARIANT,
	[0x21] = INAT_VARIANT,
	[0x22] = INAT_VARIANT,
	[0x23] = INAT_VARIANT,
	[0x24] = INAT_VARIANT,
	[0x25] = INAT_VARIANT,
	[0x28] = INAT_VARIANT,
	[0x29] = INAT_VARIANT,
	[0x2a] = INAT_VARIANT,
	[0x2b] = INAT_VARIANT,
	[0x2c] = INAT_VARIANT,
	[0x2d] = INAT_VARIANT,
	[0x2e] = INAT_VARIANT,
	[0x2f] = INAT_VARIANT,
	[0x30] = INAT_VARIANT,
	[0x31] = INAT_VARIANT,
	[0x32] = INAT_VARIANT,
	[0x33] = INAT_VARIANT,
	[0x34] = INAT_VARIANT,
	[0x35] = INAT_VARIANT,
	[0x36] = INAT_VARIANT,
	[0x37] = INAT_VARIANT,
	[0x38] = INAT_VARIANT,
	[0x39] = INAT_VARIANT,
	[0x3a] = INAT_VARIANT,
	[0x3b] = INAT_VARIANT,
	[0x3c] = INAT_VARIANT,
	[0x3d] = INAT_VARIANT,
	[0x3e] = INAT_VARIANT,
	[0x3f] = INAT_VARIANT,
	[0x40] = INAT_VARIANT,
	[0x41] = INAT_VARIANT,
	[0x45] = INAT_VARIANT,
	[0x46] = INAT_VARIANT,
	[0x47] = INAT_VARIANT,
	[0x58] = INAT_VARIANT,
	[0x59] = INAT_VARIANT,
	[0x5a] = INAT_VARIANT,
	[0x78] = INAT_VARIANT,
	[0x79] = INAT_VARIANT,
	[0x80] = INAT_VARIANT,
	[0x81] = INAT_VARIANT,
	[0x82] = INAT_VARIANT,
	[0x8c] = INAT_VARIANT,
	[0x8e] = INAT_VARIANT,
	[0x90] = INAT_VARIANT,
	[0x91] = INAT_VARIANT,
	[0x92] = INAT_VARIANT,
	[0x93] = INAT_VARIANT,
	[0x96] = INAT_VARIANT,
	[0x97] = INAT_VARIANT,
	[0x98] = INAT_VARIANT,
	[0x99] = INAT_VARIANT,
	[0x9a] = INAT_VARIANT,
	[0x9b] = INAT_VARIANT,
	[0x9c] = INAT_VARIANT,
	[0x9d] = INAT_VARIANT,
	[0x9e] = INAT_VARIANT,
	[0x9f] = INAT_VARIANT,
	[0xa6] = INAT_VARIANT,
	[0xa7] = INAT_VARIANT,
	[0xa8] = INAT_VARIANT,
	[0xa9] = INAT_VARIANT,
	[0xaa] = INAT_VARIANT,
	[0xab] = INAT_VARIANT,
	[0xac] = INAT_VARIANT,
	[0xad] = INAT_VARIANT,
	[0xae] = INAT_VARIANT,
	[0xaf] = INAT_VARIANT,
	[0xb6] = INAT_VARIANT,
	[0xb7] = INAT_VARIANT,
	[0xb8] = INAT_VARIANT,
	[0xb9] = INAT_VARIANT,
	[0xba] = INAT_VARIANT,
	[0xbb] = INAT_VARIANT,
	[0xbc] = INAT_VARIANT,
	[0xbd] = INAT_VARIANT,
	[0xbe] = INAT_VARIANT,
	[0xbf] = INAT_VARIANT,
	[0xdb] = INAT_VARIANT,
	[0xdc] = INAT_VARIANT,
	[0xdd] = INAT_VARIANT,
	[0xde] = INAT_VARIANT,
	[0xdf] = INAT_VARIANT,
	[0xf0] = INAT_MODRM | INAT_MODRM | INAT_VARIANT,
	[0xf1] = INAT_MODRM | INAT_MODRM | INAT_VARIANT,
	[0xf2] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xf3] = INAT_MAKE_GROUP(23),
	[0xf5] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY | INAT_VARIANT,
	[0xf6] = INAT_VARIANT,
	[0xf7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY | INAT_VARIANT,
};
const insn_attr_t inat_escape_table_2_1[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_MODRM | INAT_VEXOK,
	[0x01] = INAT_MODRM | INAT_VEXOK,
	[0x02] = INAT_MODRM | INAT_VEXOK,
	[0x03] = INAT_MODRM | INAT_VEXOK,
	[0x04] = INAT_MODRM | INAT_VEXOK,
	[0x05] = INAT_MODRM | INAT_VEXOK,
	[0x06] = INAT_MODRM | INAT_VEXOK,
	[0x07] = INAT_MODRM | INAT_VEXOK,
	[0x08] = INAT_MODRM | INAT_VEXOK,
	[0x09] = INAT_MODRM | INAT_VEXOK,
	[0x0a] = INAT_MODRM | INAT_VEXOK,
	[0x0b] = INAT_MODRM | INAT_VEXOK,
	[0x0c] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x0d] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x0e] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x0f] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x10] = INAT_MODRM,
	[0x13] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x14] = INAT_MODRM,
	[0x15] = INAT_MODRM,
	[0x16] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x17] = INAT_MODRM | INAT_VEXOK,
	[0x18] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x19] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x1a] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x1c] = INAT_MODRM | INAT_VEXOK,
	[0x1d] = INAT_MODRM | INAT_VEXOK,
	[0x1e] = INAT_MODRM | INAT_VEXOK,
	[0x20] = INAT_MODRM | INAT_VEXOK,
	[0x21] = INAT_MODRM | INAT_VEXOK,
	[0x22] = INAT_MODRM | INAT_VEXOK,
	[0x23] = INAT_MODRM | INAT_VEXOK,
	[0x24] = INAT_MODRM | INAT_VEXOK,
	[0x25] = INAT_MODRM | INAT_VEXOK,
	[0x28] = INAT_MODRM | INAT_VEXOK,
	[0x29] = INAT_MODRM | INAT_VEXOK,
	[0x2a] = INAT_MODRM | INAT_VEXOK,
	[0x2b] = INAT_MODRM | INAT_VEXOK,
	[0x2c] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x2d] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x2e] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x2f] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x30] = INAT_MODRM | INAT_VEXOK,
	[0x31] = INAT_MODRM | INAT_VEXOK,
	[0x32] = INAT_MODRM | INAT_VEXOK,
	[0x33] = INAT_MODRM | INAT_VEXOK,
	[0x34] = INAT_MODRM | INAT_VEXOK,
	[0x35] = INAT_MODRM | INAT_VEXOK,
	[0x36] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x37] = INAT_MODRM | INAT_VEXOK,
	[0x38] = INAT_MODRM | INAT_VEXOK,
	[0x39] = INAT_MODRM | INAT_VEXOK,
	[0x3a] = INAT_MODRM | INAT_VEXOK,
	[0x3b] = INAT_MODRM | INAT_VEXOK,
	[0x3c] = INAT_MODRM | INAT_VEXOK,
	[0x3d] = INAT_MODRM | INAT_VEXOK,
	[0x3e] = INAT_MODRM | INAT_VEXOK,
	[0x3f] = INAT_MODRM | INAT_VEXOK,
	[0x40] = INAT_MODRM | INAT_VEXOK,
	[0x41] = INAT_MODRM | INAT_VEXOK,
	[0x45] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x46] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x47] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x58] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x59] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x5a] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x78] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x79] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x80] = INAT_MODRM,
	[0x81] = INAT_MODRM,
	[0x82] = INAT_MODRM,
	[0x8c] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x8e] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x90] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x91] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x92] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x93] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x96] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x97] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x98] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x99] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9a] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9b] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9c] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9d] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9e] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x9f] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xa6] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xa7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xa8] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xa9] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xaa] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xab] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xac] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xad] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xae] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xaf] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xb6] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xb7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xb8] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xb9] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xba] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xbb] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xbc] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xbd] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xbe] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xbf] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xdb] = INAT_MODRM | INAT_VEXOK,
	[0xdc] = INAT_MODRM | INAT_VEXOK,
	[0xdd] = INAT_MODRM | INAT_VEXOK,
	[0xde] = INAT_MODRM | INAT_VEXOK,
	[0xdf] = INAT_MODRM | INAT_VEXOK,
	[0xf0] = INAT_MODRM,
	[0xf1] = INAT_MODRM,
	[0xf6] = INAT_MODRM,
	[0xf7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
};
const insn_attr_t inat_escape_table_2_2[INAT_OPCODE_TABLE_SIZE] = {
	[0xf5] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xf6] = INAT_MODRM,
	[0xf7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
};
const insn_attr_t inat_escape_table_2_3[INAT_OPCODE_TABLE_SIZE] = {
	[0xf0] = INAT_MODRM | INAT_MODRM,
	[0xf1] = INAT_MODRM | INAT_MODRM,
	[0xf5] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xf6] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0xf7] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
};

/* Table: 3-byte opcode 2 (0x0f 0x3a) */
const insn_attr_t inat_escape_table_3[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_VARIANT,
	[0x01] = INAT_VARIANT,
	[0x02] = INAT_VARIANT,
	[0x04] = INAT_VARIANT,
	[0x05] = INAT_VARIANT,
	[0x06] = INAT_VARIANT,
	[0x08] = INAT_VARIANT,
	[0x09] = INAT_VARIANT,
	[0x0a] = INAT_VARIANT,
	[0x0b] = INAT_VARIANT,
	[0x0c] = INAT_VARIANT,
	[0x0d] = INAT_VARIANT,
	[0x0e] = INAT_VARIANT,
	[0x0f] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x14] = INAT_VARIANT,
	[0x15] = INAT_VARIANT,
	[0x16] = INAT_VARIANT,
	[0x17] = INAT_VARIANT,
	[0x18] = INAT_VARIANT,
	[0x19] = INAT_VARIANT,
	[0x1d] = INAT_VARIANT,
	[0x20] = INAT_VARIANT,
	[0x21] = INAT_VARIANT,
	[0x22] = INAT_VARIANT,
	[0x38] = INAT_VARIANT,
	[0x39] = INAT_VARIANT,
	[0x40] = INAT_VARIANT,
	[0x41] = INAT_VARIANT,
	[0x42] = INAT_VARIANT,
	[0x44] = INAT_VARIANT,
	[0x46] = INAT_VARIANT,
	[0x4a] = INAT_VARIANT,
	[0x4b] = INAT_VARIANT,
	[0x4c] = INAT_VARIANT,
	[0x60] = INAT_VARIANT,
	[0x61] = INAT_VARIANT,
	[0x62] = INAT_VARIANT,
	[0x63] = INAT_VARIANT,
	[0xdf] = INAT_VARIANT,
	[0xf0] = INAT_VARIANT,
};
const insn_attr_t inat_escape_table_3_1[INAT_OPCODE_TABLE_SIZE] = {
	[0x00] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x01] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x02] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x04] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x05] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x06] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x08] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x09] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0a] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0b] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0c] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0d] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0e] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x0f] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x14] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x15] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x16] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x17] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x18] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x19] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x1d] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x20] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x21] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x22] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x38] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x39] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x40] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x41] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x42] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x44] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x46] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x4a] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x4b] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x4c] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x60] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x61] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x62] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x63] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0xdf] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
};
const insn_attr_t inat_escape_table_3_3[INAT_OPCODE_TABLE_SIZE] = {
	[0xf0] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
};

/* GrpTable: Grp1 */

/* GrpTable: Grp1A */

/* GrpTable: Grp2 */

/* GrpTable: Grp3_1 */
const insn_attr_t inat_group_table_6[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
	[0x4] = INAT_MODRM,
	[0x5] = INAT_MODRM,
	[0x6] = INAT_MODRM,
	[0x7] = INAT_MODRM,
};

/* GrpTable: Grp3_2 */
const insn_attr_t inat_group_table_7[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
	[0x4] = INAT_MODRM,
	[0x5] = INAT_MODRM,
	[0x6] = INAT_MODRM,
	[0x7] = INAT_MODRM,
};

/* GrpTable: Grp4 */
const insn_attr_t inat_group_table_8[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
};

/* GrpTable: Grp5 */
const insn_attr_t inat_group_table_9[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
	[0x2] = INAT_MODRM | INAT_FORCE64,
	[0x3] = INAT_MODRM,
	[0x4] = INAT_MODRM | INAT_FORCE64,
	[0x5] = INAT_MODRM,
	[0x6] = INAT_MODRM | INAT_FORCE64,
};

/* GrpTable: Grp6 */
const insn_attr_t inat_group_table_10[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
	[0x4] = INAT_MODRM,
	[0x5] = INAT_MODRM,
};

/* GrpTable: Grp7 */
const insn_attr_t inat_group_table_11[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
	[0x4] = INAT_MODRM,
	[0x6] = INAT_MODRM,
	[0x7] = INAT_MODRM,
};

/* GrpTable: Grp8 */

/* GrpTable: Grp9 */
const insn_attr_t inat_group_table_22[INAT_GROUP_TABLE_SIZE] = {
	[0x1] = INAT_MODRM,
	[0x6] = INAT_MODRM | INAT_MODRM | INAT_VARIANT,
	[0x7] = INAT_MODRM | INAT_MODRM | INAT_VARIANT,
};
const insn_attr_t inat_group_table_22_1[INAT_GROUP_TABLE_SIZE] = {
	[0x6] = INAT_MODRM,
};
const insn_attr_t inat_group_table_22_2[INAT_GROUP_TABLE_SIZE] = {
	[0x6] = INAT_MODRM,
	[0x7] = INAT_MODRM,
};

/* GrpTable: Grp10 */

/* GrpTable: Grp11A */
const insn_attr_t inat_group_table_4[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM,
	[0x7] = INAT_MAKE_IMM(INAT_IMM_BYTE),
};

/* GrpTable: Grp11B */
const insn_attr_t inat_group_table_5[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MAKE_IMM(INAT_IMM_VWORD32) | INAT_MODRM,
	[0x7] = INAT_MAKE_IMM(INAT_IMM_VWORD32),
};

/* GrpTable: Grp12 */
const insn_attr_t inat_group_table_14[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
};
const insn_attr_t inat_group_table_14_1[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
};

/* GrpTable: Grp13 */
const insn_attr_t inat_group_table_15[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
};
const insn_attr_t inat_group_table_15_1[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x4] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
};

/* GrpTable: Grp14 */
const insn_attr_t inat_group_table_16[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x3] = INAT_VARIANT,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VARIANT,
	[0x7] = INAT_VARIANT,
};
const insn_attr_t inat_group_table_16_1[INAT_GROUP_TABLE_SIZE] = {
	[0x2] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x3] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x6] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
	[0x7] = INAT_MAKE_IMM(INAT_IMM_BYTE) | INAT_MODRM | INAT_VEXOK,
};

/* GrpTable: Grp15 */
const insn_attr_t inat_group_table_19[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_VARIANT,
	[0x1] = INAT_VARIANT,
	[0x2] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
	[0x3] = INAT_MODRM | INAT_VEXOK | INAT_VARIANT,
};
const insn_attr_t inat_group_table_19_2[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
};

/* GrpTable: Grp16 */
const insn_attr_t inat_group_table_13[INAT_GROUP_TABLE_SIZE] = {
	[0x0] = INAT_MODRM,
	[0x1] = INAT_MODRM,
	[0x2] = INAT_MODRM,
	[0x3] = INAT_MODRM,
};

/* GrpTable: Grp17 */
const insn_attr_t inat_group_table_23[INAT_GROUP_TABLE_SIZE] = {
	[0x1] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x2] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
	[0x3] = INAT_MODRM | INAT_VEXOK | INAT_VEXONLY,
};

/* GrpTable: GrpP */

/* GrpTable: GrpPDLK */

/* GrpTable: GrpRNG */

/* Escape opcode map array */
const insn_attr_t * const inat_escape_tables[INAT_ESC_MAX + 1][INAT_LSTPFX_MAX + 1] = {
	[1][0] = inat_escape_table_1,
	[1][1] = inat_escape_table_1_1,
	[1][2] = inat_escape_table_1_2,
	[1][3] = inat_escape_table_1_3,
	[2][0] = inat_escape_table_2,
	[2][1] = inat_escape_table_2_1,
	[2][2] = inat_escape_table_2_2,
	[2][3] = inat_escape_table_2_3,
	[3][0] = inat_escape_table_3,
	[3][1] = inat_escape_table_3_1,
	[3][3] = inat_escape_table_3_3,
};

/* Group opcode map array */
const insn_attr_t * const inat_group_tables[INAT_GRP_MAX + 1][INAT_LSTPFX_MAX + 1] = {
	[4][0] = inat_group_table_4,
	[5][0] = inat_group_table_5,
	[6][0] = inat_group_table_6,
	[7][0] = inat_group_table_7,
	[8][0] = inat_group_table_8,
	[9][0] = inat_group_table_9,
	[10][0] = inat_group_table_10,
	[11][0] = inat_group_table_11,
	[13][0] = inat_group_table_13,
	[14][0] = inat_group_table_14,
	[14][1] = inat_group_table_14_1,
	[15][0] = inat_group_table_15,
	[15][1] = inat_group_table_15_1,
	[16][0] = inat_group_table_16,
	[16][1] = inat_group_table_16_1,
	[19][0] = inat_group_table_19,
	[19][2] = inat_group_table_19_2,
	[22][0] = inat_group_table_22,
	[22][1] = inat_group_table_22_1,
	[22][2] = inat_group_table_22_2,
	[23][0] = inat_group_table_23,
};

/* AVX opcode map array */
const insn_attr_t * const inat_avx_tables[X86_VEX_M_MAX + 1][INAT_LSTPFX_MAX + 1] = {
	[1][0] = inat_escape_table_1,
	[1][1] = inat_escape_table_1_1,
	[1][2] = inat_escape_table_1_2,
	[1][3] = inat_escape_table_1_3,
	[2][0] = inat_escape_table_2,
	[2][1] = inat_escape_table_2_1,
	[2][2] = inat_escape_table_2_2,
	[2][3] = inat_escape_table_2_3,
	[3][0] = inat_escape_table_3,
	[3][1] = inat_escape_table_3_1,
	[3][3] = inat_escape_table_3_3,
};
